@@ -1,8 +1,8 @@
 package JIRA::REST;
 {
-  $JIRA::REST::VERSION = '0.004';
+  $JIRA::REST::VERSION = '0.005';
 }
-# ABSTRACT: A thin wrapper around JIRA's REST API
+# ABSTRACT: Thin wrapper around JIRA's REST API
 
 use 5.010;
 use utf8;
@@ -72,6 +72,42 @@ sub new {
     } => $class;
 }
 
+sub _error {
+    my ($self, $content, $type, $code) = @_;
+
+    $type = 'text/plain' unless $type;
+    $code = 500          unless $code;
+
+    my $msg = __PACKAGE__ . " Error[$code";
+
+    if (eval {require HTTP::Status}) {
+        if (my $status = HTTP::Status::status_message($code)) {
+            $msg .= " - $status";
+        }
+    }
+
+    $msg .= "]:\n";
+
+    if ($type =~ m:text/plain:i) {
+        $msg .= $content;
+    } elsif ($type =~ m:application/json:) {
+        my $error = $self->{json}->decode($content);
+        if (ref $error eq 'HASH' && exists $error->{errorMessages}) {
+            foreach my $message (@{$error->{errorMessages}}) {
+                $msg .= "- $message\n";
+            }
+        } else {
+            $msg .= $content;
+        }
+    } elsif ($type =~ m:text/html:i && eval {require HTML::TreeBuilder}) {
+        $msg .= HTML::TreeBuilder->new_from_content($content)->as_text;
+    } else {
+        $msg .= "<unconvertable Content-Type: '$type}'>";
+    };
+    $msg =~ s/\n*$//s;       # strip trailing newlines
+    return $msg;
+}
+
 sub _content {
     my ($self) = @_;
 
@@ -80,30 +116,26 @@ sub _content {
     my $type    = $rest->responseHeader('Content-Type');
     my $content = $rest->responseContent();
 
-    unless ($code =~ /^2/) {
-        my $message = eval {require HTTP::Status}
-            ? HTTP::Status::status_message($code) || '(unknown)'
-                : '(?)';
-        croak "ERROR: $code - $message\n$type\n$content\n";
-    }
+    $code =~ /^2/
+        or croak $self->_error($content, $type, $code);
 
     return unless $content;
 
     if (! defined $type) {
-        croak "Cannot convert response content with no Content-Type specified.\n";
+        croak $self->_error("Cannot convert response content with no Content-Type specified.");
     } elsif ($type =~ m:^application/json:i) {
         return $self->{json}->decode($content);
     } elsif ($type =~ m:^text/plain:i) {
         return $content;
     } else {
-        croak "I don't understand content with Content-Type '$type'.\n";
+        croak $self->_error("I don't understand content with Content-Type '$type'.");
     }
 }
 
 sub _build_query {
-    my ($query) = @_;
+    my ($self, $query) = @_;
 
-    is_hash_ref($query) or croak "The QUERY argument must be a hash-ref.\n";
+    is_hash_ref($query) or croak $self->_error("The QUERY argument must be a hash-ref.");
 
     return '?'. join('&', map {$_ . '=' . uri_escape($query->{$_})} keys %$query);
 }
@@ -111,7 +143,7 @@ sub _build_query {
 sub GET {
     my ($self, $path, $query) = @_;
 
-    $path .= _build_query($query) if $query;
+    $path .= $self->_build_query($query) if $query;
 
     $self->{rest}->GET($path);
 
@@ -121,7 +153,7 @@ sub GET {
 sub DELETE {
     my ($self, $path, $query) = @_;
 
-    $path .= _build_query($query) if $query;
+    $path .= $self->_build_query($query) if $query;
 
     $self->{rest}->DELETE($path);
 
@@ -131,7 +163,7 @@ sub DELETE {
 sub PUT {
     my ($self, $path, $query, $value, $headers) = @_;
 
-    $path .= _build_query($query) if $query;
+    $path .= $self->_build_query($query) if $query;
 
     $headers                   //= {};
     $headers->{'Content-Type'} //= 'application/json;charset=UTF-8';
@@ -144,7 +176,7 @@ sub PUT {
 sub POST {
     my ($self, $path, $query, $value, $headers) = @_;
 
-    $path .= _build_query($query) if $query;
+    $path .= $self->_build_query($query) if $query;
 
     $headers                   //= {};
     $headers->{'Content-Type'} //= 'application/json;charset=UTF-8';
@@ -154,6 +186,46 @@ sub POST {
     return $self->_content();
 }
 
+ sub set_search_iterator {
+    my ($self, $params) = @_;
+
+    my %params = ( %$params );  # rebuild the hash to own it
+
+    $params{startAt}    = 0;
+    $params{maxResults} = 256
+        unless exists $params{maxResults};
+
+    my $results = $self->POST('/search', \%params);
+
+    $self->{iter} = {
+        params  => \%params,    # params hash used in the previous call
+        results => $results,    # issues returned by the last call
+        offset  => 0,           # offset to be used in the next call
+    };
+
+    return;
+}
+
+sub next_issue {
+    my ($self) = @_;
+
+    my $iter = $self->{iter}
+        or croak $self->_error("You must call set_search_iterator before calling next_issue");
+
+    if ($iter->{offset} == $iter->{results}{total}) {
+        if ($iter->{results}{startAt} + $iter->{offset} == $iter->{results}{maxResults}) {
+            $self->{iter} = undef;
+            return undef;
+        } else {
+            $iter->{params}{startAt} = $iter->{results}{startAt} + $iter->{offset};
+            $iter->{results}         = $self->POST('/search', $iter->{params});
+            $iter->{offset}          = 0;
+        }
+    }
+
+    return $iter->{results}{issues}[$iter->{offset}++];
+}
+
 1;
 
 __END__
@@ -164,11 +236,11 @@ __END__
 
 =head1 NAME
 
-JIRA::REST - A thin wrapper around JIRA's REST API
+JIRA::REST - Thin wrapper around JIRA's REST API
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -176,26 +248,12 @@ version 0.004
 
     my $jira = JIRA::REST->new('https://jira.example.net', 'myuser', 'mypass');
 
-    # Cache some JIRA objects
-    my %issuetypes = map {($_->{name} => $_)} @{$jira->GET('/issuetype')};
-    my %priorities = map {($_->{name} => $_)} @{$jira->GET('/priority' )};
-    my %projects   = map {($_->{name} => $_)} @{$jira->GET('/project'  )};
-
-    # A handy routine to construct id hashes
-    sub id_of { return { id => $_[0]->{id} }; };
-
-    # Cache information about a project
-    my $pid        = $projects{'My Project'}->{id};
-    my %components = map {($_->{name} => $_)} @{$jira->GET("/project/$pid/components")};
-
     # File a bug
     my $issue = $jira->POST('/issue', undef, {
         fields => {
-            project   => id_of($projects{'My Project'}),
-            issuetype => id_of($issuetypes{Bug}),
+            project   => { key => 'PRJ' },
+            issuetype => { name => 'Bug' },
             summary   => 'Cannot login',
-            priority  => id_of($priorities{Major}),
-            versions  => [ id_of($components{auth_module}) ],
             description => 'Bla bla bla',
         },
     });
@@ -215,6 +273,17 @@ version 0.004
         print "Found issue $issue->{key}\n";
     }
 
+    # Iterate using utility methods
+    $jira->set_search_iterator({
+        jql        => 'project = "TST" and status = "open"',
+        maxResults => 16,
+        fields     => [ qw/summary status assignee/ ],
+    });
+
+    while (my $issue = $jira->next_issue) {
+        print "Found issue $issue->{key}\n";
+    }
+
 =head1 DESCRIPTION
 
 L<JIRA|http://www.atlassian.com/software/jira/> is a proprietary bug
@@ -269,7 +338,7 @@ overwrites any value associated with the C<host> key in this hash.
 
 =back
 
-=head1 METHODS
+=head1 REST METHODS
 
 JIRA's REST API documentation lists dozens of "resources" which can be
 operated via the standard HTTP requests: GET, DELETE, PUT, and
@@ -381,6 +450,30 @@ Creates RESOURCE based on VALUE.
 
 Updates RESOURCE based on VALUE.
 
+=head1 UTILITY METHODS
+
+This module provides a few utility methods.
+
+=head2 B<set_search_iterator> PARAMS
+
+Sets up an iterator for the search specified by the hash-ref PARAMS. It must
+be called before calls to B<next_issue>.
+
+PARAMS must conform with the query parameters allowed for the
+C</rest/api/2/search> JIRA REST endpoint. It may specify a C<maxResults>
+parameter. If not, a default of 256 will be used.
+
+=head2 B<next_issue>
+
+This must be called after a call to B<set_search_iterator>. Each call
+returns a reference to the next issue from the filter. When there are no
+more issues it returns undef.
+
+Using the set_search_iterator/next_issue utility methods you can iterate
+through large sets of issues without worrying about the startAt/total/offset
+attributes in the response from the /search REST endpoint. These methods
+implement the "paging" algorithm needed to work with those attributes.
+
 =head1 SEE ALSO
 
 =over
@@ -409,7 +502,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2014 by CPqD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
